<?php

namespace FiloBlu\Flow\Model\Actions\From;

class Addcrosssell {

    CONST ENTITY_TYPE_CODE = 'catalog_product';

    protected $_helper_product;
    protected $_product_factory;
    /**
    * @var Monolog\Logger
    */
    protected $_logger;

    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \FiloBlu\Flow\Helper\Product $helper_product,
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        array $data = []
    ) {
        $this->_helper_product = $helper_product;
        $this->_product_factory = $productFactory;
        $this->_logger = $loggerProvider->getLogger();
    }

    public function process($product, $value, $storeId) {
        
       if (empty($value)) {
            return true;
        }

        \Magento\Framework\App\ObjectManager::getInstance()->get('Magento\Store\Model\StoreManagerInterface')->setCurrentStore($storeId);
        if ($storeId != 0) {
            return "The function is avalaible only for the ADMIN store.";
        }

        // controllo se si tratta di più related con separatore |
        $_crosssell_sku = array();
        if(strrpos($value, '|')) {
            $_crosssell_sku = explode( '|', $value);
        } else {
            $_crosssell_sku[] = $value;
        }
        if(is_array($_crosssell_sku) && count($_crosssell_sku)>=1) {

            $linkDataAll = array();
            
            // trovo i prodotti che hanno relazioni diverse da 'crosssell' e li conservo: non devono essere sovrascritti          
            $links = $product->getProductLinks();
            foreach ($links as $link) {
                if($link['link_type'] != 'crosssell') {
                    $linkDataAll[] = $link;
                }
            }

            foreach($_crosssell_sku as $_crosssell) {

                //check first if the product exist
                $linkedProduct = $this->_product_factory->create()->getIdBySku($_crosssell);
                if($linkedProduct) {
                   /** @var  \Magento\Catalog\Api\Data\ProductLinkInterface $productLinks */
                   $productLinks = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Catalog\Api\Data\ProductLinkInterface');
                   $linkData = $productLinks
                       ->setSku($product->getSku())
                       ->setLinkedProductSku($_crosssell)
                       ->setLinkType("crosssell");
                   $linkDataAll[] = $linkData;
               }

            }

            if($linkDataAll) {
                $product->setProductLinks($linkDataAll);
            }
         $product->save();

        }



        // mettere in configurazione se i related devono essere salvati anche su configurabile - default YES
        if ($product->getTypeId() == \Magento\Catalog\Model\Product\Type::DEFAULT_TYPE) {
            $parentIds = \Magento\Framework\App\ObjectManager::getInstance()->get('Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable')
                ->getParentIdsByChild($product->getId());
            if (count($parentIds) <= 0)
                return true;

            foreach ($parentIds as $parentId) {
                $parent_product = $this->_product_factory->create()->load($parentId);

                if(is_array($_crosssell_sku) && count($_crosssell_sku)>=1) {

                    $linkDataAll = array();
                    
                    // trovo i prodotti che hanno relazioni diverse da 'crosssell' e li conservo: non devono essere sovrascritti          
                    $links = $parent_product->getProductLinks();
                    foreach ($links as $link) {
                        if($link['link_type'] != 'crosssell') {
                            $linkDataAll[] = $link;
                        }
                    }

                    foreach($_crosssell_sku as $_crosssell) {

                        $linkedProduct = $this->_product_factory->create()->getIdBySku($_crosssell);
                        if($linkedProduct) {
                           /** @var  \Magento\Catalog\Api\Data\ProductLinkInterface $productLinks */
                           $productLinks = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Catalog\Api\Data\ProductLinkInterface');
                           $linkData = $productLinks
                               ->setSku($parent_product->getSku())
                               ->setLinkedProductSku($_crosssell)
                               ->setLinkType("crosssell");
                           $linkDataAll[] = $linkData;
                        }

                    }

                    if($linkDataAll) {
                        $parent_product->setProductLinks($linkDataAll);
                    }
                 $parent_product->save();

                }
            }

        }

    }

}
